@@ -78,3 +78,13 @@ class AttributeConflict(OctueSDKException):
     """Raise if, when trying to set an attribute whose current value has a significantly higher confidence than the new
     value, the new value conflicts with the current value.
     """
+
+
+class MissingServiceID(OctueSDKException):
+    """Raise when a specific ID for a service is expected to be provided, but is missing or None."""
+
+
+class CloudLocationNotSpecified(OctueSDKException):
+    """Raise when attempting to interact with a cloud resource implicitly but the implicit details of its location are
+    missing.
+    """
