@@ -4,20 +4,9 @@ from octue import exceptions
 def _filter(instance, filter_name=None, filter_value=None):
     """Returns a new instance containing only the Filterables to which the given filter criteria apply.
 
-    Say we want to filter by files whose extension equals "csv". We want to be able to do...
-
-    ds = DataSet(... initialise with csv files and other files)
-    Example of chaining:
-
-    ds.filter('files__extension_equals', 'csv')
-    is equvalent to
-    ds.files.filter(extension__equals', 'csv')
-    >>> FilterSet containing a set of datafiles
-
-
-    ds.filter('files__extension_equals', 'csv')
-    is equvalent to
-    ds.files.filter(extension__equals', 'csv')
+    :param str filter_name:
+    :param any filter_value:
+    :return octue.resources.filter_containers.FilterSet:
     """
     return instance.__class__((item for item in instance if item.satisfies(filter_name, filter_value)))
 
